/// <reference path="../../definitions/vso-node-api/vso-node-api.d.ts"/>
var apim = require('vso-node-api/WebApi');
/*
 * formats lists of optional and required arguments for a command into a standard output for the command help
 * eg: 'args: <required1> <required2> [--optional1 <optional1>] [--optional2 <optional2>]'
 * @param requiredArguments an array of the names of all arguments that are required
 * @param requiredArguments an array of the names of all arguments that can be optionally provided
 * @param flags boolean flags (optional because less frequently used) eg "--all"
 */
function formatArgumentsHint(requiredArguments, optionalArguments, flags) {
    var argumentsHint = "";
    for (var i = 0; i < requiredArguments.length; i++) {
        argumentsHint += ' <' + requiredArguments[i] + '>';
    }
    for (var i = 0; i < optionalArguments.length; i++) {
        argumentsHint += ' [--' + optionalArguments[i] + ' <' + optionalArguments[i] + '>]';
    }
    argumentsHint += ' [options]';
    return argumentsHint;
}
exports.formatArgumentsHint = formatArgumentsHint;
var TfCommand = (function () {
    function TfCommand() {
    }
    // setConnection
    // getWebApi() 
    TfCommand.prototype.getWebApi = function () {
        return new apim.WebApi(this.connection.collectionUrl, this.connection.authHandler);
    };
    //
    // should return a JSON data object which will be
    // - printed in json if --json, or
    // - passed back to output for readable text
    //
    TfCommand.prototype.exec = function (args, options) {
        var defer = Q.defer();
        defer.reject(new Error('Not implemented.  Must override'));
        return defer.promise;
    };
    TfCommand.prototype.output = function (data) {
        // should override and output to console results
        // in readable text based on data from exec call
    };
    /*
     * throws an error if a required argument was not provided with a command
     */
    TfCommand.prototype.checkRequiredParameter = function (parameterValue, parameterName, friendlyName) {
        var finalFriendlyName = friendlyName || parameterName;
        if (!parameterValue) {
            throw new Error('Required parameter ' + parameterName + ' not supplied. Try adding a switch to the end of your command: --' + parameterName + ' <' + finalFriendlyName + '>');
        }
    };
    return TfCommand;
})();
exports.TfCommand = TfCommand;
